/*
 * file: window_lift.c
 * This file is part of the libpv library examples.
 *
 * Copyright 2005 Pengutronix
 * Juergen Beisert j.beisert@pengutronix.de
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 *
 * PURPOSE:
 * This is the master side of this demo. It emulates
 * the real hardware. The real hardware is a motor,
 * button up, down, switch for end position, photo sensor
 * and some leds.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <termios.h>

#include <libpv.h>

static int loop=1;

static int8_t *button_up;
static int8_t *button_down;
static uint32_t *photo_sensor;
static int8_t *end_switch;
static int8_t *motor_up;
static int8_t *motor_down;

#define M_ON 1
#define M_OFF 0
#define M_UP 1
#define M_DOWN 0

static int motor; /* M_ON or M_OFF */
static int dir;	/* M_UP or M_DOWN */
static int motor_position;	/* 0 (bottom) ... 100 (top) */
static struct termios stored_settings;

/**
 * to stop the endless loops
 **/
void signalhandler (int sig)
{
	printf("Signal received. Terminating\n");
	if (sig == SIGTERM || sig == SIGINT)
		loop=0;	/* end all loops */
}

/**
 * Thread to create motor timing
 **/
static void *motor_thread(void *p)
{
	pv_engine_t *pv_engine=(pv_engine_t*)p;
	struct timespec tsp = {
		.tv_sec=0,
		.tv_nsec=125000000	/* 1/8 sec */
	};

	while (loop) {
		nanosleep(&tsp,NULL);
		if (motor == M_ON) {
			if (dir == M_UP) {
				/*
				 * move up until 100%
				 */
				motor_position++;
				if (motor_position >= 100) {
					motor_position=100;
					pv_lock(pv_engine);
					*end_switch = 1;
					pv_event_generate(pv_engine,end_switch);
					pv_unlock(pv_engine);
				}
				else {
					pv_lock(pv_engine);
					*photo_sensor ^= 1;	/* its moving */
					pv_event_generate(pv_engine,photo_sensor);
					if (*end_switch != 0) {
						*end_switch = 0;
						pv_event_generate(pv_engine,end_switch);
					}
					pv_unlock(pv_engine);
				}
			}
			else {
				/*
				 * move down until 0%
				 */
				motor_position--;
				if (motor_position <= 0) {
					motor_position=0;
					pv_lock(pv_engine);
					*end_switch = 1;
					pv_event_generate(pv_engine,end_switch);
					pv_unlock(pv_engine);
				}
				else {
					*photo_sensor ^= 1;	/* its moving */
					pv_event_generate(pv_engine,photo_sensor);
					pv_lock(pv_engine);
					if (*end_switch != 0) {
						*end_switch = 0;
						pv_event_generate(pv_engine,end_switch);
					}
					pv_unlock(pv_engine);
				}
			}
		}
	}

	return(NULL);
}

/**
 * Reset TTY to standard behaviour
 **/
static void resetTTY(void)
{
   tcsetattr(0,TCSANOW,&stored_settings);
}

/**
 * Change TTY to deliver each hit key asap
 **/
static int setToPressOnly(void)
{
struct termios new_settings;

   tcgetattr(0,&stored_settings);

   new_settings=stored_settings;

/* Disable canonical mode and set buffer size to 1 byte */

   new_settings.c_lflag &= (~ICANON);
   new_settings.c_cc[VTIME]=0;
   new_settings.c_cc[VMIN]=1;

   tcsetattr(0,TCSANOW,&new_settings);

   return(1);
}

/**
 * Thread to notify about keyboard hits
 **/
static void *tty_thread(void *p)
{
	pv_engine_t *pv_engine=(pv_engine_t*)p;
	int c;

	setToPressOnly();	/* to get each key asap */
	while(loop) {
		c=getchar();
		switch (c) {
		case 'u':
		case 'U':
			pv_lock(pv_engine);
			*button_up ^= 1;	/* FIXME: That's not perfect */
			if (*button_down != 0) {
				*button_down=0;
				pv_event_generate(pv_engine,button_down);
			}
			pv_event_generate(pv_engine,button_up);
			pv_unlock(pv_engine);
			break;

		case 'd':
		case 'D':
			pv_lock(pv_engine);
			*button_down ^= 1;	/* FIXME: That's not perfect */
			if (*button_up != 0) {
				*button_up=0;
				pv_event_generate(pv_engine,button_up);
			}
			pv_event_generate(pv_engine,button_down);
			pv_unlock(pv_engine);
			break;
		}
	}

	resetTTY();	/* standard behaviour */
	return(NULL);
}

int main(int argc,char *argv[])
{
	pv_engine_attr_t	attr;
	pv_engine_t			pv_engine;
	void				*var;
	pthread_t			thread1=0,thread2=0;

	if (argc > 1) {
		/*
		* parse the given parameters
		*/
		if (pv_engine_attr_init_from_cmdline(&attr, argc, argv))
			return(EXIT_FAILURE);
	}
	else {
		/*
		 * Without any parameters setup default values
		 */
		pv_engine_attr_init(&attr);
		strcpy(attr.desc_filename,argv[0]);
		strcat(attr.desc_filename,".xml");
		strcpy(attr.store_filename,argv[0]);
		strcat(attr.desc_filename,".bin");
		attr.jvisu_port=0;
	}

	/*
	 * establish connection to the process image
	 */
	if (pv_init(&pv_engine, &attr))
		return(EXIT_FAILURE);

	/*
	 * Register our variables
	 */
	if ((button_up=pv_register_int8(&pv_engine,"button1")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*button_up=0;
	if ((button_down=pv_register_int8(&pv_engine,"button2")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*button_down=0;
	if ((photo_sensor=pv_register_uint32(&pv_engine,"timestamp")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*photo_sensor=0;
	if ((end_switch=pv_register_int8(&pv_engine,"button3")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*end_switch=0;
	if ((motor_up=pv_register_int8(&pv_engine,"motor1")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*motor_up=0;
	if ((motor_down=pv_register_int8(&pv_engine,"motor2")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	*motor_down=0;

	/*
	 * What we are doing here:
	 * - We emulate the buttons, photo sensor and switch here
	 * - we wait for notifications on motor signals
	 */
	pv_event_register(&pv_engine,motor_up);
	pv_event_register(&pv_engine,motor_down);

/*	signal(SIGTERM, signalhandler);
	signal(SIGINT, signalhandler); */

	/*
	 * Two threads creates timings and interaction
	 */
	pthread_create(&thread1,NULL,motor_thread,&pv_engine);
	pthread_create(&thread2,NULL,tty_thread,&pv_engine);

	while (loop) {
		pv_event_await_syncronous(&pv_engine,&var);
		pv_lock(&pv_engine);
		if (var == motor_up) {
			if (*motor_up == 0)
				motor=M_OFF;
			else {
				motor=M_ON;
				dir=M_UP;
			}
		}
		if (var == motor_down) {
			if (*motor_down == 0)
				motor=M_OFF;
			else {
				motor=M_ON;
				dir=M_DOWN;
			}
		}
		pv_unlock(&pv_engine);
	}

	if (pthread_cancel(thread1) == 0)	/* only to be sure */
		pthread_join(thread1,NULL);
	if (pthread_cancel(thread2) == 0)	/* only to be sure */
		pthread_join(thread2,NULL);

exit_1:
	pv_destroy(&pv_engine);
	return(EXIT_SUCCESS);
}

/*
#
# vim settings
#
# vim:set ts=4:
# vim:set shiftwidth=4:
# vim:set nowrap:
#
*/

/* end if file window_lift.c */
